@@ -2,12 +2,12 @@ from __future__ import annotations
 
 import ast
 import dataclasses
+import difflib
 import inspect
 import textwrap
 from collections import defaultdict
 from typing import TYPE_CHECKING, Callable, Dict, List, Mapping, NoReturn, Optional, Type, overload
 
-import Levenshtein
 from executing import Source
 
 from chalk._lsp._class_finder import get_function_ast
@@ -46,6 +46,8 @@ from chalk.parsed.duplicate_input_gql import (
     PositionGQL,
     RangeGQL,
 )
+from chalk.utils.collections import OrderedSet
+from chalk.utils.source_parsing import should_skip_source_code_parsing
 from chalk.utils.string import oxford_comma_list
 
 if TYPE_CHECKING:
@@ -154,15 +156,17 @@ class LSPErrorBuilder:
             i += 1
         if frame is None:
             return
-        try:
-            node = Source.executing(frame).node
-        except Exception:
-            return
-        if node is not None:
+        node_map_key = (wrapper, item)
+        if node_map_key not in cls._node_map:
             try:
-                cls._node_map[(wrapper, item)] = (node, frame)
-            except:
-                pass
+                node = Source.executing(frame).node
+            except Exception:
+                return
+            if node is not None:  # pyright: ignore[reportUnnecessaryComparison]
+                try:
+                    cls._node_map[node_map_key] = (node, frame)
+                except:
+                    pass
 
     @classmethod
     def get_node(cls, wrapper: FeatureWrapper, item: str) -> tuple[ast.AST, types.FrameType] | None:
@@ -197,6 +201,7 @@ class FeatureClassErrorBuilder:
         self.diagnostics: List[DiagnosticGQL] = []
         self.namespace = namespace
         self.node = node
+        self.error_cache: OrderedSet[tuple[str, RangeGQL | ast.AST, str]] = OrderedSet()
 
     def property_range(self, feature_name: str) -> ast.AST | None:
         if self.node is None:
@@ -235,10 +240,19 @@ class FeatureClassErrorBuilder:
         return get_class_definition_range(cls=self.node, filename=self.uri)
 
     def invalid_attribute(
-        self, namespace: str, item: str, candidates: List[str], back: int, saved_frame: FeatureWrapper | None = None
+        self,
+        root_feature_str: str,
+        root_is_feature_class: bool,
+        item: str,
+        candidates: List[str],
+        back: int,
+        saved_frame: FeatureWrapper | None = None,
     ):
         back = back + 1
-        message = f"Invalid attribute '{item}' on feature class '{namespace}'."
+        message = (
+            f"Invalid attribute '{item}' on feature {'class ' if root_is_feature_class else ''}"
+            + f"'{root_feature_str}'."
+        )
         if not LSPErrorBuilder.lsp:
             # Short circuit if we're not in an LSP context. What follows is expensive.
             raise AttributeError(message)
@@ -282,12 +296,12 @@ class FeatureClassErrorBuilder:
         if len(candidates) > 0:
             all_scores = [
                 (
-                    Levenshtein.distance(item, candidate),
+                    difflib.SequenceMatcher(a=item, b=candidate).quick_ratio(),
                     candidate,
                 )
                 for candidate in candidates
             ]
-            all_scores.sort(key=lambda x: x[0])
+            all_scores.sort(key=lambda x: -x[0])
 
             if len(candidates) > 5:
                 prefix = "The closest options are"
@@ -314,6 +328,7 @@ class FeatureClassErrorBuilder:
         message: str,
         label: str,
         code: str,
+        *,
         range: RangeGQL | ast.AST | None,
         code_href: str | None = None,
         severity: DiagnosticSeverityGQL = DiagnosticSeverityGQL.Error,
@@ -328,10 +343,11 @@ class FeatureClassErrorBuilder:
         message: str,
         label: str,
         code: str,
+        *,
         range: RangeGQL | ast.AST | None,
         code_href: str | None = None,
         severity: DiagnosticSeverityGQL = DiagnosticSeverityGQL.Error,
-        raise_error: Type[Exception] = ...,
+        raise_error: Type[Exception],
         uri: str | None = ...,
     ) -> NoReturn:
         ...
@@ -371,15 +387,14 @@ class FeatureClassErrorBuilder:
         )
 
         error = None if raise_error is None else raise_error(message)
-        if range is not None:
-            if error is None:
+        if error is None:
+            if (message, range, uri) not in self.error_cache:
                 self.diagnostics.append(builder.diagnostic)
                 LSPErrorBuilder.all_errors[uri].append(builder.diagnostic)
-            else:
-                LSPErrorBuilder.save_exception(error, uri, builder.diagnostic)
-                raise error
+                self.error_cache.add((message, range, uri))
         else:
-            raise error or default_error(message)
+            LSPErrorBuilder.save_exception(error, uri, builder.diagnostic)
+            raise error
 
         return builder
 
@@ -412,7 +427,7 @@ class ResolverErrorBuilder:
     def _load_node_and_uri(self):
         """Lazily loading the node and uri on first use, because parsing the source is slow. If there are no errors, then no need to parse it"""
         source_info: Optional[FunctionSource] = None
-        if self._fn is not None:
+        if not should_skip_source_code_parsing() and self._fn is not None:
             try:
                 filename = inspect.getfile(self._fn)
                 resolver_source = inspect.getsource(self._fn)
@@ -453,6 +468,7 @@ class ResolverErrorBuilder:
         message: str,
         label: str,
         code: str,
+        *,
         range: RangeGQL | ast.AST | None,
         code_href: str | None = None,
         severity: DiagnosticSeverityGQL = DiagnosticSeverityGQL.Error,
@@ -509,16 +525,12 @@ class ResolverErrorBuilder:
         )
 
         error = None if raise_error is None else raise_error(message)
-        if range is not None:
-            if error is None:
-                self.diagnostics.append(builder.diagnostic)
-                LSPErrorBuilder.all_errors[uri].append(builder.diagnostic)
-            else:
-                LSPErrorBuilder.save_exception(error, uri, builder.diagnostic)
-                raise error
+        if error is None:
+            self.diagnostics.append(builder.diagnostic)
+            LSPErrorBuilder.all_errors[uri].append(builder.diagnostic)
         else:
-            raise error or default_error(message)
-
+            LSPErrorBuilder.save_exception(error, uri, builder.diagnostic)
+            raise error
         return builder
 
     def function_decorator(self) -> ast.AST | None:
@@ -652,9 +664,10 @@ def get_resolver_error_builder(fn: Callable) -> ResolverErrorBuilder:
 
 
 class SQLFileResolverErrorBuilder:
-    def __init__(self, uri: str, sql_string: str):
+    def __init__(self, uri: str, sql_string: str, has_import_errors: bool):
         super().__init__()
         self.uri = uri
+        self.has_import_errors = has_import_errors
         self.diagnostics: List[DiagnosticGQL] = []
         self.sql_string = sql_string
         self.sql_lines = sql_string.splitlines()
@@ -682,6 +695,9 @@ class SQLFileResolverErrorBuilder:
         :param uri: filepath
         :return:
         """
+        if self.has_import_errors:
+            # pass: we don't need lsp
+            return _dummy_builder
         if not LSPErrorBuilder.lsp:
             if raise_error is not None:
                 raise raise_error(message)
@@ -707,15 +723,12 @@ class SQLFileResolverErrorBuilder:
         )
 
         error = None if raise_error is None else raise_error(message)
-        if range is not None:
-            if error is None:
-                self.diagnostics.append(builder.diagnostic)
-                LSPErrorBuilder.all_errors[uri].append(builder.diagnostic)
-            else:
-                LSPErrorBuilder.save_exception(error, uri, builder.diagnostic)
-                raise error
+        if error is None:
+            self.diagnostics.append(builder.diagnostic)
+            LSPErrorBuilder.all_errors[uri].append(builder.diagnostic)
         else:
-            raise error or default_error(message)
+            LSPErrorBuilder.save_exception(error, uri, builder.diagnostic)
+            raise error
 
         return builder
 
@@ -780,12 +793,12 @@ class SQLFileResolverErrorBuilder:
         if len(candidates) > 0:
             all_scores = [
                 (
-                    Levenshtein.distance(spellcheck_item, candidate),
+                    difflib.SequenceMatcher(a=spellcheck_item, b=candidate).quick_ratio(),
                     candidate,
                 )
                 for candidate in candidates
             ]
-            all_scores.sort(key=lambda x: x[0])
+            all_scores.sort(key=lambda x: -x[0])
 
             if len(candidates) > 5:
                 prefix = "The closest options are"
@@ -821,7 +834,7 @@ def range_or_node_to_start_line(range_or_node: RangeGQL | ast.AST) -> int | None
     if isinstance(range_or_node, RangeGQL):
         i = range_or_node.start.line
     else:
-        i = range_or_node.lineno
+        i = getattr(range_or_node, "lineno", None)
         if i is None:
             return None
     return i - 1
@@ -831,7 +844,7 @@ def range_or_node_to_end_line(range_or_node: RangeGQL | ast.AST) -> int | None:
     if isinstance(range_or_node, RangeGQL):
         i = range_or_node.end.line
     else:
-        i = range_or_node.end_lineno
+        i = getattr(range_or_node, "end_lineno", None)
         if i is None:
             return None
     return i - 1
@@ -841,7 +854,7 @@ def range_or_node_to_start_char(range_or_node: RangeGQL | ast.AST) -> int | None
     if isinstance(range_or_node, RangeGQL):
         i = range_or_node.start.character
     else:
-        i = range_or_node.col_offset
+        i = getattr(range_or_node, "col_offset", None)
         if i is None:
             return None
     return i
@@ -851,7 +864,7 @@ def range_or_node_to_end_char(range_or_node: RangeGQL | ast.AST) -> int | None:
     if isinstance(range_or_node, RangeGQL):
         i = range_or_node.end.character
     else:
-        i = range_or_node.end_col_offset
+        i = getattr(range_or_node, "end_col_offset", None)
         if i is None:
             return None
     return max(i - 1, 0)
