@@ -33,6 +33,7 @@ from chalk.client.models import (
 )
 from chalk.client.response import Dataset, OnlineQueryResult
 from chalk.features import DataFrame, Feature
+from chalk.ml.model_file_transfer import SourceConfig
 
 if TYPE_CHECKING:
     import ssl
@@ -2142,6 +2143,7 @@ class ChalkClient:
         metadata: Optional[Mapping[str, Any]] = None,
         input_features: Optional[list[str]] = None,
         output_features: Optional[list[str]] = None,
+        source_config: Optional[SourceConfig] = None,
     ) -> RegisterModelVersionResponse:
         """
         Register a model in the Chalk model registry.
@@ -2177,6 +2179,8 @@ class ChalkClient:
             The features to be used as outputs to the model.
             For example, `[User.is_spam]`. Features can also be expressed as snakecased strings,
             e.g. `["user.is_spam"]`
+        source_config : LocalSourceConfig, S3SourceConfig, HFSourceConfig, optional
+            Config to pass credentials to access files from a remote source.
         Returns
         -------
         ModelVersion
