@@ -3,22 +3,17 @@ from __future__ import annotations
 import collections.abc
 import dataclasses
 import datetime as dt
-import hashlib
 import json
-import os
 import random
-import tempfile
 import typing
 import warnings
 from functools import cached_property
-from math import ceil
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Literal, Mapping, Optional, Sequence, Tuple, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Callable, List, Literal, Mapping, Optional, Sequence, Tuple, TypeVar, Union
 from urllib.parse import urlparse
 
 import grpc
 import grpc.experimental
-import requests
-from google.protobuf import empty_pb2, struct_pb2, timestamp_pb2
+from google.protobuf import empty_pb2, timestamp_pb2
 
 from chalk import DataFrame, EnvironmentId, chalk_logger
 from chalk._gen.chalk.auth.v1.agent_pb2 import CustomClaim
@@ -93,6 +88,7 @@ from chalk.client.models import (
     ResourceRequests,
     UploadFeaturesResponse,
 )
+from chalk.client.serialization.model_serialization import ModelSerializer
 from chalk.client.serialization.protos import ChalkErrorConverter, OnlineQueryConverter, UploadFeaturesBulkConverter
 from chalk.config.auth_config import TokenConfig, load_token
 from chalk.features import live_updates
@@ -102,7 +98,8 @@ from chalk.features._encoding.outputs import encode_outputs
 from chalk.features.feature_set import is_feature_set_class
 from chalk.features.tag import DeploymentId
 from chalk.importer import CHALK_IMPORT_FLAG
-from chalk.ml import ModelEncoding, ModelType
+from chalk.ml import ModelEncoding, ModelType, SourceConfig
+from chalk.ml.model_file_transfer import ModelFileUploader
 from chalk.parsed._proto.utils import datetime_to_proto_timestamp, value_to_proto
 from chalk.utils import df_utils
 from chalk.utils.df_utils import record_batch_to_arrow_ipc
@@ -1316,122 +1313,6 @@ class ChalkGRPCClient:
             except grpc.RpcError as e:
                 raise RuntimeError(f"Could not register model version. {e.details()}")
 
-    @staticmethod
-    def _build_tabular_schema(column_dtypes: Mapping[str, Any]) -> _model_artifact_pb2.TabularSchema:
-        """
-        Build a TabularSchema from a dictionary of column names to dtypes.
-
-        Parameters
-        ----------
-        column_dtypes : dict
-            Dictionary mapping column names to their data types.
-            Data types can be PyArrow types or Python types (str, int, float, bool).
-
-        Returns
-        -------
-        TabularSchema
-            A protobuf TabularSchema object
-
-        Examples
-        --------
-        >>> import pyarrow as pa
-        >>> schema = ChalkGRPCClient._build_tabular_schema({
-        ...     'alcohol': pa.float64(),
-        ...     'malic_acid': pa.float64(),
-        ...     'ash': pa.float64(),
-        ... })
-        """
-        import pyarrow as pa
-
-        from chalk.features._encoding.converter import PrimitiveFeatureConverter
-
-        tabular_schema = _model_artifact_pb2.TabularSchema()
-
-        for col_name, dtype in column_dtypes.items():
-            # Convert to PyArrow type if needed
-            if not isinstance(dtype, pa.DataType):
-                # Handle Python types
-                if dtype == str:
-                    pa_dtype = pa.string()
-                elif dtype == int:
-                    pa_dtype = pa.int64()
-                elif dtype == float:
-                    pa_dtype = pa.float64()
-                elif dtype == bool:
-                    pa_dtype = pa.bool_()
-                else:
-                    raise ValueError(f"Unsupported dtype {dtype} for column {col_name}")
-            else:
-                pa_dtype = dtype
-
-            # Convert PyArrow dtype to protobuf Arrow type
-            proto_arrow_type = PrimitiveFeatureConverter.convert_pa_dtype_to_proto_dtype(pa_dtype)
-
-            # Create TabularSpec for this column
-            column_spec = _model_artifact_pb2.TabularSpec(name=col_name, dtype=proto_arrow_type)
-
-            tabular_schema.columns.append(column_spec)
-
-        return tabular_schema
-
-    @staticmethod
-    def _build_tensor_schema(tensor_specs: List[Tuple[List[int], Any]]) -> _model_artifact_pb2.TensorSchema:
-        """
-        Build a TensorSchema from a list of (shape, dtype) pairs.
-
-        Parameters
-        ----------
-        tensor_specs : list of tuple
-            List of (shape, dtype) tuples where:
-            - shape is a list of integers representing tensor dimensions
-            - dtype is a PyArrow type or Python type (str, int, float, bool)
-
-        Returns
-        -------
-        TensorSchema
-            A protobuf TensorSchema object
-
-        Examples
-        --------
-        >>> import pyarrow as pa
-        >>> schema = ChalkGRPCClient._build_tensor_schema([
-        ...     ([224, 224, 3], pa.float32()),  # RGB image tensor
-        ...     ([100], pa.float64()),          # Feature vector
-        ... ])
-        """
-        import pyarrow as pa
-
-        from chalk.features._encoding.converter import PrimitiveFeatureConverter
-
-        tensor_schema = _model_artifact_pb2.TensorSchema()
-
-        for shape, dtype in tensor_specs:
-            # Convert to PyArrow type if needed
-            if not isinstance(dtype, pa.DataType):
-                # Handle Python types
-                if dtype == str:
-                    pa_dtype = pa.string()
-                elif dtype == int:
-                    pa_dtype = pa.int64()
-                elif dtype == float:
-                    pa_dtype = pa.float64()
-                elif dtype == bool:
-                    pa_dtype = pa.bool_()
-                else:
-                    raise ValueError(f"Unsupported dtype {dtype} for tensor")
-            else:
-                pa_dtype = dtype
-
-            # Convert PyArrow dtype to protobuf Arrow type
-            proto_arrow_type = PrimitiveFeatureConverter.convert_pa_dtype_to_proto_dtype(pa_dtype)
-
-            # Create TensorSpec
-            tensor_spec = _model_artifact_pb2.TensorSpec(dtype=proto_arrow_type, shape=shape)
-
-            tensor_schema.tensors.append(tensor_spec)
-
-        return tensor_schema
-
     def register_model(
         self,
         name: str,
@@ -1471,17 +1352,8 @@ class ChalkGRPCClient:
         ...     }
         ... )
         """
-        metadata_converted: Dict[str, struct_pb2.Value] = {}
 
-        for k, v in metadata.items():
-            converted_v = struct_pb2.Value()
-            if isinstance(v, str):
-                converted_v.string_value = v
-            elif isinstance(v, (int, float)):
-                converted_v.number_value = v
-            elif isinstance(v, bool):
-                converted_v.bool_value = v
-            metadata_converted[k] = converted_v
+        metadata_converted = ModelSerializer.convert_metadata_to_protobuf(metadata)
 
         try:
             resp: CreateModelResponse = self._stub_refresher.call_model_stub(
@@ -1493,7 +1365,6 @@ class ChalkGRPCClient:
                     )
                 )
             )
-
             return RegisterModelResponse(
                 model_id=resp.model.id,
                 model_name=resp.model.model_name,
@@ -1531,6 +1402,7 @@ class ChalkGRPCClient:
         metadata: Optional[Mapping[str, Any]] = None,
         input_features: Optional[list[str]] = None,
         output_features: Optional[list[str]] = None,
+        source_config: Optional[SourceConfig] = None,
     ) -> RegisterModelVersionResponse:
         """
         Register a model in the Chalk model registry.
@@ -1570,6 +1442,8 @@ class ChalkGRPCClient:
             The features to be used as outputs to the model.
             For example, `[User.is_spam]`. Features can also be expressed as snakecased strings,
             e.g. `["user.is_spam"]`
+        source_config : LocalSourceConfig, S3SourceConfig, HFSourceConfig, optional
+            Config to pass credentials to access files from a remote source.
         Returns
         -------
         ModelVersion
@@ -1582,7 +1456,7 @@ class ChalkGRPCClient:
         >>> client.register_model_version(
         ...     name="RiskModel",
         ...     model=trained_sklearn_model,
-        ...     model_type="pytorch",
+        ...     model_type=ModelType.PYTORCH,
         ... )
 
         Register from local files:
@@ -1593,7 +1467,7 @@ class ChalkGRPCClient:
         >>> client.register_model_version(
         ...     name="RiskModel",
         ...     model_paths=["./model.pth"],
-        ...     model_type="pytorch",
+        ...     model_type=ModelType.PYTORCH,
         ...     input_schema=pa.large_string(),
         ...     output_schema=pa.float32()
         ... )
@@ -1603,279 +1477,105 @@ class ChalkGRPCClient:
         >>> client.register_model_version(
         ...     name="RiskModel",
         ...     model_paths=["s3://my-bucket/path/to/model.pth"],
-        ...     model_type="pytorch",
+        ...     model_type=ModelType.PYTORCH,
         ... )
         """
-        model_upload_paths: List[_model_artifact_pb2.ModelFile] = []
-        additional_file_upload_path: List[_model_artifact_pb2.ModelFile] = []
-        temp_files_to_cleanup: List[str] = []
+        with ModelSerializer.from_model(model, model_type) as model_serializer:
+            model_file_uploader = ModelFileUploader(source_config)
 
-        if input_schema is None and input_features is None:
-            raise ValueError(
-                "You must specify at least one of (input_schema, input_features) to register a model version."
-            )
-        if output_schema is None and output_features is None:
-            raise ValueError(
-                "You must specify at least one of (output_schema, output_features) to register a model version."
-            )
+            if model is not None:
+                inferred_input_schema, inferred_output_schema = model_serializer.infer_input_output_schemas(
+                    model, model_type
+                )
+                if input_schema is None and inferred_input_schema is not None:
+                    input_schema = inferred_input_schema
+                if output_schema is None and inferred_output_schema is not None:
+                    output_schema = inferred_output_schema
 
-        try:
-            metadata_converted: Dict[str, struct_pb2.Value] = {}
-            if metadata is not None:
-                for k, v in metadata.items():
-                    converted_v = struct_pb2.Value()
-                    if isinstance(v, str):
-                        converted_v.string_value = v
-                    elif isinstance(v, (int, float)):
-                        converted_v.number_value = v
-                    elif isinstance(v, bool):
-                        converted_v.bool_value = v
-                    metadata_converted[k] = converted_v
-
-            if model_paths is None:
-                if model is None:
-                    raise ValueError("Failed to register model. Please specify a model or model_paths.")
-
-                tmp_dir = tempfile.mkdtemp()
-
-                if model_type == ModelType.PYTORCH:
-                    try:
-                        import torch
-                    except ImportError:
-                        raise ImportError("Please install PyTorch to save PyTorch models.")
-                    model_path = os.path.join(tmp_dir, "model.pth")
-
-                    torch.save(model, model_path)  # Save entire model to support torch.load()
-
-                    model_encoding = ModelEncoding.PICKLE
-
-                elif model_type == ModelType.SKLEARN:
-                    try:
-                        import joblib
-                    except ImportError:
-                        raise ImportError("Please install joblib to save sklearn models.")
-                    model_path = os.path.join(tmp_dir, "model.pkl")
-                    joblib.dump(model, model_path)
-                    model_encoding = ModelEncoding.PICKLE
-
-                elif model_type == ModelType.TENSORFLOW:
-                    model_path = os.path.join(tmp_dir, "model.h5")
-                    model.save(model_path)
-                    model_encoding = ModelEncoding.HDF5
-
-                elif model_type == ModelType.XGBOOST:
-                    model_path = os.path.join(tmp_dir, "model.json")
-                    model.save_model(model_path)
-                    model_encoding = ModelEncoding.JSON
-
-                elif model_type == ModelType.LIGHTGBM:
-                    model_path = os.path.join(tmp_dir, "model.txt")
-                    model.save_model(model_path)
-                    model_encoding = ModelEncoding.TEXT
-
-                elif model_type == ModelType.CATBOOST:
-                    model_path = os.path.join(tmp_dir, "model.cbm")
-                    model.save_model(model_path)
-                    model_encoding = ModelEncoding.CBM
-
-                elif model_type == ModelType.ONNX:
-                    model_path = os.path.join(tmp_dir, "model.onnx")
-                    model.save_model(model_path)
-                    model_encoding = ModelEncoding.PROTOBUF
-
-                else:
-                    raise NotImplementedError(
-                        f"Unsupported model type: {model_type}; Please save the model to a file and register with "
-                        + "explicit model_paths or contact Chalk Support."
-                    )
-
-                model_paths = [model_path]
-                temp_files_to_cleanup.append(model_path)
-
-            else:
-                if model is not None:
-                    raise ValueError(
-                        "Failed to register model. Ambiguous model, can't specify both model_paths and model."
-                    )
-                if model_encoding is None:
-                    raise ValueError("Failed to register model. Please specify a model encoding if using model_paths.")
-
-            input_model_schema = _model_artifact_pb2.ModelSchema()
-            if input_schema is not None:
-                if isinstance(input_schema, dict):
-                    input_model_schema.tabular.CopyFrom(self._build_tabular_schema(input_schema))
-                elif isinstance(input_schema, list):
-                    input_model_schema.tensor.CopyFrom(self._build_tensor_schema(input_schema))
-                else:
-                    raise ValueError(f"Invalid input_schema: {input_schema}")
-
-            output_model_schema = _model_artifact_pb2.ModelSchema()
-            if output_schema is not None:
-                if isinstance(output_schema, dict):
-                    output_model_schema.tabular.CopyFrom(self._build_tabular_schema(output_schema))
-                elif isinstance(output_schema, list):
-                    output_model_schema.tensor.CopyFrom(self._build_tensor_schema(output_schema))
-                else:
-                    raise ValueError(f"Invalid output_schema: {output_schema}")
-
-            all_files_to_process: Dict[str, str] = {}  # map of filename -> local_path
-            model_file_names: List[str] = []
-
-            for model_path in model_paths:
-                filename = os.path.basename(model_path)
-                all_files_to_process[filename] = model_path
-                model_file_names.append(filename)
-
-            additional_file_mapping: Dict[str, str] = {}  # map of filename -> file_type
-            if additional_files is not None:
-                for file_type, file_path in additional_files.items():
-                    filename = os.path.basename(file_path)
-                    all_files_to_process[filename] = file_path
-                    additional_file_mapping[filename] = file_type
-
-            presigned_s3_response: ModelUploadUrlResponse = self._get_model_artifact_presigned_s3(
-                model_paths=list(all_files_to_process.keys())
-            )
+            if input_schema is None:
+                raise ValueError("You must specify at least one of input_schema to register a model version.")
+            if output_schema is None and output_features is None:
+                raise ValueError("You must specify at least one of output_schema to register a model version.")
 
             try:
-                import boto3
-            except ImportError:
-                raise ImportError("Please install boto3 to enable model registration.")
+                metadata_converted = model_serializer.convert_metadata_to_protobuf(metadata)
 
-            aws_profile = os.environ.get("AWS_PROFILE")
-            aws_region = os.environ.get("AWS_REGION")
-
-            session = boto3.Session(profile_name=aws_profile, region_name=aws_region)
-            s3_client = session.client("s3")
-
-            for filename, local_or_s3_path in all_files_to_process.items():
-                presigned_url = presigned_s3_response.upload_urls[filename]
-                parsed_path = urlparse(local_or_s3_path)
-
-                filesize_kb: int
-                file_hash: bytes
-
-                if parsed_path.scheme == "s3":
-                    src_bucket = parsed_path.netloc
-                    src_key = parsed_path.path.lstrip("/")
-                    try:
-                        response = s3_client.get_object(Bucket=src_bucket, Key=src_key)
-                        file_data = response["Body"].read()
-
-                        file_hash = hashlib.sha256(file_data).digest()
-                        filesize_kb = ceil(response["ContentLength"] / 1024.0)
-
-                        put_response = requests.put(
-                            presigned_url,
-                            data=file_data,
-                            headers={"Content-Type": response.get("ContentType", "application/octet-stream")},
-                        )
-
-                        if put_response.status_code != 200:
-                            raise RuntimeError(
-                                f"Failed to upload to presigned URL for {filename}: "
-                                + f"{put_response.status_code} {put_response.text}"
-                            )
-                    except Exception as e:
-                        raise RuntimeError(f"Unable to get object from {local_or_s3_path}. {e}")
-                elif parsed_path.scheme == "" or parsed_path.scheme == "file":
-
-                    def _validate_local_path(path: str):
-                        abs_path = os.path.abspath(path)
-                        if ".." in os.path.relpath(abs_path, start=os.getcwd()).split(os.sep):
-                            if not os.path.commonpath([abs_path, tmp_dir]) == tmp_dir:
-                                raise ValueError(f"Unsafe file path: {path}")
-                        if not os.path.isfile(abs_path):
-                            raise FileNotFoundError(f"Local file not found: {path}")
-                        return abs_path
-
-                    try:
-                        safe_path = _validate_local_path(local_or_s3_path)
-                        with open(safe_path, "rb") as f:
-                            file_data = f.read()
-
-                        file_hash = hashlib.sha256(file_data).digest()
-                        filesize_kb = ceil(os.path.getsize(local_or_s3_path) / 1024.0)
-
-                        put_response = requests.put(presigned_url, data=file_data)
-
-                        if put_response.status_code != 200:
-                            raise RuntimeError(
-                                f"Failed to upload local file {local_or_s3_path} to presigned URL: "
-                                + f"{put_response.status_code} {put_response.text}"
-                            )
-
-                    except Exception as e:
-                        raise RuntimeError(f"Failed to upload local file {local_or_s3_path}: {e}")
+                tmp_path = ""
+                if model_paths is None:
+                    if model is None:
+                        raise ValueError("Failed to register model. Please specify a model or model_paths.")
+                    tmp_path, model_encoding = model_serializer.serialize_model(model, model_type)
+                    model_paths = [tmp_path]
                 else:
-                    raise ValueError(f"Unsupported file path format: {local_or_s3_path}")
+                    if model is not None:
+                        raise ValueError(
+                            "Failed to register model. Ambiguous model, can't specify both model_paths and model."
+                        )
+                    if model_encoding is None:
+                        raise ValueError(
+                            "Failed to register model. Please specify a model encoding if using model_paths."
+                        )
 
-                if filename in model_file_names:
-                    model_upload_paths.append(
-                        _model_artifact_pb2.ModelFile(
-                            name=filename,
-                            size_kb=filesize_kb,
-                            file_hash=file_hash,
-                        )
-                    )
-                else:
-                    additional_file_upload_path.append(
-                        _model_artifact_pb2.ModelFile(
-                            name=filename,
-                            size_kb=filesize_kb,
-                            file_hash=file_hash,
-                        )
-                    )
-            try:
-                resp: CreateModelVersionResponse = self._stub_refresher.call_model_stub(
-                    lambda x: x.CreateModelVersion(
-                        CreateModelVersionRequest(
-                            model_name=name,
-                            model_artifact_id=presigned_s3_response.model_artifact_id,
-                            model_artifact=_model_artifact_pb2.ModelArtifactSpec(
-                                model_files=model_upload_paths,
-                                additional_files=additional_file_upload_path,
-                                model_type=model_type,
-                                model_encoding=model_encoding,
-                                model_signature=_model_artifact_pb2.ModelSignature(
-                                    inputs=input_model_schema,
-                                    outputs=output_model_schema,
-                                ),
-                                input_features=input_features,
-                                output_features=output_features,
-                            ),
-                            aliases=aliases,
-                            metadata=metadata_converted,
-                        )
-                    )
+                input_model_schema = model_serializer.convert_schema(input_schema)
+                output_model_schema = model_serializer.convert_schema(output_schema)
+
+                all_files_to_process, model_file_names = model_file_uploader.prepare_file_mapping(
+                    model_paths, additional_files
                 )
-                return RegisterModelVersionResponse(
-                    model_id=resp.model_version.id,
-                    model_name=resp.model_version.model_name,
-                    model_version=resp.model_version.version,
-                    artifact=resp.model_version.model_artifact,
-                    aliases=list(resp.model_version.aliases),
-                    metadata=dict(resp.model_version.metadata),
-                    created_by=resp.model_version.created_by,
-                    created_at=resp.model_version.created_at.ToDatetime(),
+
+                presigned_s3_response: ModelUploadUrlResponse = self._get_model_artifact_presigned_s3(
+                    model_paths=list(all_files_to_process.keys())
                 )
-            except grpc.RpcError as e:
-                raise RuntimeError(f"Could not register model version. {e.details()}")
 
-        finally:
-            import shutil
-
-            for temp_path in temp_files_to_cleanup:
+                model_upload_paths, additional_files_upload_paths = model_file_uploader.upload_files(
+                    all_files_to_process,
+                    model_file_names,
+                    presigned_s3_response.upload_urls,
+                    [tmp_path],
+                )
                 try:
-                    if os.path.isfile(temp_path):
-                        os.remove(temp_path)
-                    elif os.path.isdir(temp_path):
-                        shutil.rmtree(temp_path)
-                    temp_dir = os.path.dirname(temp_path)
-                    if temp_dir and os.path.exists(temp_dir) and temp_dir.startswith(tempfile.gettempdir()):
-                        shutil.rmtree(temp_dir)
-                except Exception:
-                    pass
+                    resp: CreateModelVersionResponse = self._stub_refresher.call_model_stub(
+                        lambda x: x.CreateModelVersion(
+                            CreateModelVersionRequest(
+                                model_name=name,
+                                model_artifact_id=presigned_s3_response.model_artifact_id,
+                                model_artifact=_model_artifact_pb2.ModelArtifactSpec(
+                                    model_files=[
+                                        model_serializer.fileinfo_to_protobuf(file) for file in model_upload_paths
+                                    ],
+                                    additional_files=[
+                                        model_serializer.fileinfo_to_protobuf(file)
+                                        for file in additional_files_upload_paths
+                                    ],
+                                    model_type=model_type,
+                                    model_encoding=model_encoding,
+                                    model_signature=_model_artifact_pb2.ModelSignature(
+                                        inputs=input_model_schema,
+                                        outputs=output_model_schema,
+                                    ),
+                                    input_features=input_features,
+                                    output_features=output_features,
+                                ),
+                                aliases=aliases,
+                                metadata=metadata_converted,
+                            )
+                        )
+                    )
+                    return RegisterModelVersionResponse(
+                        model_id=resp.model_version.id,
+                        model_name=resp.model_version.model_name,
+                        model_version=resp.model_version.version,
+                        artifact=resp.model_version.model_artifact,
+                        aliases=list(resp.model_version.aliases),
+                        metadata=dict(resp.model_version.metadata),
+                        created_by=resp.model_version.created_by,
+                        created_at=resp.model_version.created_at.ToDatetime(),
+                    )
+                except grpc.RpcError as e:
+                    raise RuntimeError(f"Could not register model version. {e.details()}")
+            except Exception as e:
+                raise RuntimeError(f"Could not register model version. {e}")
+        raise RuntimeError("Error creating model serializer context.")
 
     def create_model_training_job(
         self,
