@@ -9,6 +9,7 @@ from chalk.features._encoding.missing_value import MissingValueStrategy
 from chalk.features._encoding.primitive import TPrimitive, TPrimitiveArrowScalar
 from chalk.features._encoding.serialized_dtype import deserialize_dtype, serialize_dtype
 from chalk.features._geospatial import LatLon, LatLonRadians
+from chalk.features._tensor import Tensor, TensorLayout
 from chalk.features._vector import Vector
 from chalk.features.dataframe import DataFrame
 from chalk.features.feature_field import CacheStrategy, Feature, FeatureNotFoundException, feature, has_many, has_one
@@ -206,6 +207,8 @@ __all__ = [
     "TPrimitive",
     "TPrimitiveArrowScalar",
     "Tags",
+    "Tensor",
+    "TensorLayout",
     "TimeDelta",
     "Underscore",
     "Vector",
