@@ -17,13 +17,20 @@ import isodate
 
 # Need the *real* BaseModel here (not the v1 one) so that we can register the right hook
 from pydantic import BaseModel as BaseBaseModel
+
+try:
+    from pydantic.v1 import BaseModel as V1BaseModel
+except ImportError:
+    V1BaseModel = None
+
 from typing_extensions import get_args, get_origin, is_typeddict
 
 from chalk.features._encoding.primitive import ChalkStructType, TPrimitive
 from chalk.utils.cached_type_hints import cached_get_type_hints
 from chalk.utils.collections import is_namedtuple, unwrap_optional_and_annotated_if_needed
 from chalk.utils.enum import get_enum_value_type
-from chalk.utils.pydanticutil.pydantic_compat import is_pydantic_basemodel
+from chalk.utils.json import JSON
+from chalk.utils.pydanticutil.pydantic_compat import construct_pydantic_model, is_pydantic_basemodel
 
 TRich = TypeVar("TRich")
 T = TypeVar("T")
@@ -45,6 +52,10 @@ _rich_converter.register_unstructure_hook_func(
 _rich_converter.register_unstructure_hook(
     BaseBaseModel, lambda x: {k: _rich_converter.unstructure(v) for (k, v) in x.dict().items()}
 )
+if V1BaseModel is not None:
+    _rich_converter.register_unstructure_hook(
+        V1BaseModel, lambda x: {k: _rich_converter.unstructure(v) for (k, v) in x.dict().items()}
+    )
 _rich_converter.register_unstructure_hook_func(
     attrs.has,
     lambda x: {k: _rich_converter.unstructure(v) for (k, v) in attrs.asdict(x, recurse=False).items()},
@@ -76,10 +87,10 @@ def _structure_struct(obj: Any, typ: Type):
     if not isinstance(obj, collections.abc.Mapping):
         raise TypeError(f"Unable to structure object `{obj}` into type `{typ}`. Only dictionaries are supported.")
     kwargs = {k: _rich_converter.structure(v, type_hints[k]) for (k, v) in obj.items()}
-    if isinstance(typ, type) and is_pydantic_basemodel(typ):
+    if isinstance(typ, type) and is_pydantic_basemodel(typ):  # pyright: ignore[reportUnnecessaryIsInstance]
         # Using .construct to bypass pydantic validation, which could fail if there are null values in kwargs
         # and the field is not annotated as null
-        return typ.construct(**kwargs)
+        return construct_pydantic_model(typ, **kwargs)
     else:
         return typ(**kwargs)
 
@@ -280,7 +291,9 @@ def _structure_date(x: Any, typ: Type[date]) -> date:
 
 
 _rich_converter.register_structure_hook_func(
-    lambda x: isinstance(x, type) and issubclass(x, date) and not issubclass(x, datetime),
+    lambda x: isinstance(x, type)  # pyright: ignore[reportUnnecessaryIsInstance]
+    and issubclass(x, date)  # pyright: ignore[reportUnnecessaryIsInstance]
+    and not issubclass(x, datetime),
     _structure_date,
 )
 
@@ -430,12 +443,21 @@ def _structure_rich(obj: Any, typ: Type):
 
     if (
         isinstance(obj, (list, set, frozenset, dict, tuple, BaseBaseModel))
+        or (V1BaseModel is not None and isinstance(obj, V1BaseModel))
         or dataclasses.is_dataclass(obj)
         or attrs.has(type(obj))
     ):
         raise ValueError(f"Object {obj} of type {type(obj).__name__} cannot be converted into a {typ.__name__}")
 
-    return typ(obj)
+    # Perform this import lazily, to prevent a circular import.
+    from chalk.features.underscore import Underscore
+
+    if isinstance(obj, Underscore) and not issubclass(typ, Underscore):
+        raise ValueError(
+            f"Chalk expression '{repr(obj)}' cannot be provided where a concrete value of type '{typ.__name__}' is expected"
+        )
+
+    return typ(obj)  # pyright: ignore
 
 
 _rich_converter.register_structure_hook(int, _structure_rich)
@@ -458,18 +480,22 @@ def unstructure_rich_to_primitive(val: Any) -> TPrimitive:
 
 
 def structure_primitive_to_rich(val: TPrimitive, typ: Type[TRich]) -> TRich:
-    if (val is None or type(val) is typ) and typ in (
-        str,
-        int,
-        bool,
-        float,
-        bytes,
-        timedelta,
-        decimal.Decimal,
-        date,
-        datetime,
-        time,
-    ):
+    if (
+        (val is None or type(val) is typ)
+        and typ
+        in (
+            str,
+            int,
+            bool,
+            float,
+            bytes,
+            timedelta,
+            decimal.Decimal,
+            date,
+            datetime,
+            time,
+        )
+    ) or typ is JSON:  # pyright: ignore[reportUnnecessaryComparison]
         # Short circuit these rich types
         return cast(TRich, val)
     return _rich_converter.structure(val, typ)
