@@ -39,7 +39,7 @@ from chalk._validation.validation import Validation
 from chalk.features._encoding.converter import FeatureConverter, JSONCodec, TDecoder, TEncoder
 from chalk.features._encoding.primitive import TPrimitive
 from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY, FeatureRegistryProtocol
-from chalk.features.feature_wrapper import FeatureWrapper
+from chalk.features.feature_wrapper import FeatureWrapper, NearestNeighborException
 from chalk.features.filter import Filter, TimeDelta, ClauseJoinWithAndException
 from chalk.features.tag import Tags
 from chalk.features.underscore import Underscore
@@ -507,7 +507,7 @@ class Feature(Generic[_TPrim, _TRich]):
 
     @property
     def converter(self) -> FeatureConverter:
-        from chalk.features import DataFrame, Vector
+        from chalk.features import DataFrame, Vector, Tensor
 
         self._converter_entered += 1
 
@@ -704,6 +704,57 @@ class Feature(Generic[_TPrim, _TRich]):
                     f"Vector types must be PyArrow FixedSizeLists of float16, float32, or float64 values. Feature '{self.root_fqn}' is of type {pyarrow_dtype}."
                 )
 
+        tensor_typ = self.typ.as_tensor()
+        if tensor_typ is not None:
+            if decoder is not None:
+                raise ValueError(
+                    (
+                        "When using a Tensor type, the decoder cannot be manually specified. Please remove the "
+                        f"`feature(decoder=...)` argument from the feature definition for feature '{self.root_fqn}'."
+                    )
+                )
+
+            def tensor_decoder(primitive: Tensor | List[Any]) -> Tensor:
+                if isinstance(primitive, Tensor):
+                    return primitive
+                data = np.array(primitive, dtype=np.dtype(tensor_typ.dtype.to_pandas_dtype()))
+                return Tensor(data)
+
+            decoder = cast(TDecoder[_TPrim, _TRich], tensor_decoder)
+            if encoder is not None:
+                raise ValueError(
+                    (
+                        "When using a Tensor type, the encoder cannot be manually specified. Please remove the "
+                        f"`feature(encoder=...)` argument from the feature definition for feature '{self.root_fqn}'"
+                    )
+                )
+
+            def tensor_encoder(rich: Vector) -> Any:
+                return rich.to_numpy().tolist()
+
+            encoder = cast(TEncoder[_TPrim, _TRich], tensor_encoder)
+            if pyarrow_dtype is None:
+                try:
+                    pyarrow_dtype = tensor_typ.to_pyarrow_dtype()
+                except AttributeError:
+                    # The dtype property will not be set if someone does x: Tensor without specifying the type in the generics
+                    raise ValueError(
+                        (
+                            f"The tensor annotation for feature '{self.root_fqn}' is missing the number of dimensions. "
+                            f"Please update the annotation from `{self.attribute_name}: Tensor` to "
+                            f"`{self.attribute_name}: Tensor[N, ...]`, where N is a dimension."
+                        )
+                    ) from None
+
+            if (
+                not pa.types.is_fixed_size_list(pyarrow_dtype)
+                and not pa.types.is_list(pyarrow_dtype)
+                and not pa.types.is_large_list(pyarrow_dtype)
+            ):
+                raise TypeError(
+                    f"Vector types must be serialized as a PyArrow List/FixedSizeList. Feature '{self.root_fqn}' is of type {pyarrow_dtype}."
+                )
+
         feature_typ = self.typ.as_feature()
         if feature_typ is not None:
             if self._converter_entered > 1:
@@ -1310,6 +1361,19 @@ class Feature(Generic[_TPrim, _TRich]):
                 code="32",
                 raise_error=TypeError,
             )
+        except NearestNeighborException as ne:
+            assert self.features_cls is not None
+            self.lsp_error_builder.add_diagnostic(
+                message=(
+                    f"The attribute '{self.features_cls.__name__}.{self.attribute_name}' "
+                    f"has a join function that has an incorrectly configured nearest neighbor join: {ne} "
+                ),
+                label="invalid nearest neighbor join",
+                range=self.lsp_error_builder.property_value_range(self.attribute_name)
+                      or self.lsp_error_builder.property_range(self.attribute_name),
+                code="32",
+                raise_error=TypeError,
+            )
         except NameError as ne:
             object_str = ne.name
             assert self.features_cls is not None
@@ -1868,7 +1932,24 @@ def feature(
     store_offline
         By default `True`. Setting to `False` will prevent this feature from being written to the offline store.
     versions
-        A map from integer feature version to feature definition. If this argument is used, then no other argument to `features` can be used.
+        A map from integer feature version to feature definition. If this argument is used, no other argument
+        to `features` can be used.
+        >>> from chalk.features import features, feature
+        >>> @features
+        ... class User:
+        ...     id: str
+        ...     score: int
+        ...     banned: bool = feature(versions={
+        ...         1: feature(
+        ...              deprecated=True,
+        ...              description="Prior thresholds we used",
+        ...              expression=_.score > 900,
+        ...         ),
+        ...         2: feature(
+        ...              description="New cutoff released by Monica's team",
+        ...              expression=_.score > 300 | (_.score < 100 & _.id == "admin"),
+        ...         ),
+        ...     })
     deprecated
         If `True`, this feature is considered deprecated, which impacts the dashboard, alerts,
         and warnings.
