@@ -18,7 +18,6 @@ class ModelReference:
         version: int | None = None,
         alias: str | None = None,
         as_of_date: datetime | None = None,
-        load_model: bool = False,
     ):
         """Specifies the model version that should be loaded into the deployment.
 
@@ -75,12 +74,15 @@ class ModelReference:
         self.code = source_code
         self.source_line_end = source_line_end
 
-        dup_mv = MODEL_REFERENCE_REGISTRY.get((name, identifier), None)
-        if dup_mv is not None:
+        self.relations: list[tuple[list[str], str]] = []
+        self.resolvers: list[str] = []
+
+        dup_mr = MODEL_REFERENCE_REGISTRY.get((name, identifier), None)
+        if dup_mr is not None:
             self.errors.append(
                 (
                     "Model Reference must be distinct on name and identifier, but found two model bundles with name "
-                    f"'{name}' and identifier '{identifier}' in files '{dup_mv.filename}' and '{filename}'."
+                    f"'{name}' and identifier '{identifier}' in files '{dup_mr.filename}' and '{filename}'."
                 )
             )
 
@@ -95,24 +97,24 @@ class ModelReference:
                 name=name,
                 version=version,
                 as_of_date=as_of_date,
+                identifier=identifier,
                 model_type=model_type_from_proto(model_artifact_metadata.spec.model_type),
                 model_encoding=model_encoding_from_proto(model_artifact_metadata.spec.model_encoding),
             )
 
-            if load_model:
-                from chalk.features.hooks import before_all
+            from chalk.features.hooks import before_all
 
-                def hook():
-                    mv.load_model()
+            def hook():
+                mv.load_model()
 
-                before_all(hook)
+            before_all(hook)
 
             self.model_version = mv
         else:
-            self.model_version = ModelVersion(name=name)
+            self.model_version = ModelVersion(name=name, identifier=identifier)
 
     @classmethod
-    def as_of(cls, name: str, when: datetime, load_model: bool = False) -> ModelVersion:
+    def as_of(cls, name: str, when: datetime) -> ModelVersion:
         """Creates a ModelReference for a specific point in time.
 
         Parameters
@@ -134,11 +136,11 @@ class ModelReference:
         >>> model = ModelReference.as_of("fraud_model", timestamp)
         """
 
-        mr = ModelReference(name=name, as_of_date=when, load_model=load_model)
+        mr = ModelReference(name=name, as_of_date=when)
         return mr.model_version
 
     @classmethod
-    def from_version(cls, name: str, version: int, load_model: bool = False) -> ModelVersion:
+    def from_version(cls, name: str, version: int) -> ModelVersion:
         """Creates a ModelReference using a numeric version identifier.
 
         Parameters
@@ -165,11 +167,11 @@ class ModelReference:
         if version < 0:
             raise ValueError("Version number must be a non-negative integer.")
 
-        mr = ModelReference(name=name, version=version, load_model=load_model)
+        mr = ModelReference(name=name, version=version)
         return mr.model_version
 
     @classmethod
-    def from_alias(cls, name: str, alias: str, load_model: bool = False) -> ModelVersion:
+    def from_alias(cls, name: str, alias: str) -> ModelVersion:
         """Creates a ModelReference using an alias identifier.
 
         Parameters
@@ -196,7 +198,7 @@ class ModelReference:
         if not alias:
             raise ValueError("Alias must be a non-empty string.")
 
-        mr = ModelReference(name=name, alias=alias, load_model=load_model)
+        mr = ModelReference(name=name, alias=alias)
         return mr.model_version
 
 
