@@ -1287,6 +1287,18 @@ class ToProtoConverter:
         for feat, expr in (r.feature_expressions or {}).items():
             expr_proto = cls.convert_underscore(expr)
             feature_expressions[str(feat)] = graph_pb2.FeatureExpression(underscore_expr=expr_proto)
+        message_producer: pb.StreamResolverMessageProducer | None = None
+        if r.message_producer_parsed is not None:
+            message_producer = pb.StreamResolverMessageProducer(
+                send_to=ToProtoConverter.create_stream_source_reference(r.message_producer_parsed.send_to),
+                output_features=r.message_producer_parsed.output_features,
+                feature_expressions={
+                    str(k): graph_pb2.FeatureExpression(underscore_expr=cls.convert_underscore(v))
+                    for k, v in r.message_producer_parsed.feature_expressions.items()
+                }
+                if r.message_producer_parsed.feature_expressions is not None
+                else None,
+            )
 
         # convert_proto_message_type_to_pyarrow_type(global_value.DESCRIPTOR)
         explicit_schema_proto: arrow_pb.ArrowType | None = None
@@ -1328,6 +1340,7 @@ class ToProtoConverter:
                 captured_globals=r.function_captured_globals,
             ),
             feature_expressions=feature_expressions,
+            message_producer=message_producer,
         )
 
     @staticmethod
