@@ -1402,6 +1402,8 @@ def _get_stream_resolver(
             autogenerated=False,
             updates_materialized_aggregations=True,
             sql_settings=None,
+            feature_expressions=None,
+            message_producer=None,
         )
     except Exception as e:
         message = f"Streaming SQL file resolver could not be instantiated, {e}"
