@@ -1,7 +1,7 @@
-import typing
+from typing import Collection
 
 
-def tag_matches(datasource_tag: str, context_tags: typing.Set[str]) -> bool:
+def tag_matches(datasource_tag: str, context_tags: Collection[str]) -> bool:
     for t in context_tags:
         if t == datasource_tag:
             return True
